#!/usr/bin/perl -w
use strict;
use Carp;
use IO::Socket;
require 'sys/syscall.ph';
use lib $ENV{SPQRLIB};
use spqrcrypt;
use timestamp;
use packet;
my $probe;

# Da implementare: controllo della correttezza dei dati
sub check {
    return $#_;
}

$|=1;

# Estremi del server
my $port=shift @ARGV;
my $VERSION=0;
my $signature=spqrcrypt::newsalt;

defined ( $port ) || ( $port = 22000 );
my $message;

# Costruzione del socket UDP
socket(SOCKET, PF_INET, SOCK_DGRAM, getprotobyname('udp')) 
    || die "socket: $!";
setsockopt(SOCKET, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)) 
    || die "setsockopt: $!";
# bind del socket sulla porta "nota"
bind(SOCKET, sockaddr_in($port, INADDR_ANY)) 
    || die "bind: $!";

while ( my ( $probe, $t1tval ) =  tsrecv(*SOCKET, $message) ) {
    my $type = unpack("C",$message) & 0xF;
    
  SWITCH: {
      if ($type == $packet::messageType{open}) {
	  my ( $length, $period, $port) = 
	      unpack("x2nNx2n",$message);
          my ($hisport, $hisiaddr) = sockaddr_in($probe);
          my $probeIP = gethostbyaddr($hisiaddr,AF_INET) ||
	      warn "gethostbyaddr: $!";
	  my $remote = IO::Socket::INET->new(
				    Proto    => "UDP",
				    PeerAddr => $probeIP,
				    PeerPort => $port
				    )
	      || die "socket: $!";
#DEBUG: proposes a half length
#	  $length=$length/2;
	  $length=$length;
	  print "signature=$signature\n";
	  my $hispaddr = sockaddr_in($port, $hisiaddr);
	  my $VT = ( $VERSION << 4) |
	      $packet::messageType{reply};
	  my $reply = pack ("CCnNA4",
			      $VT, 
			      0x0, 
			      $length, 
			      $period,  
			      $signature);

	  defined( send ($remote, $reply, 0, $hispaddr) )
	      || die "send OPEN reply: $!";
	  warn "sent reply to $probeIP:$port";
	  last SWITCH; 
      }
      if ($type == $packet::messageType{forward}) { 
	  my ( $length, $mixed, $port, $probesignature, @data ) = 
	      unpack("x2nNx2nA4C*",$message);
	  check(@data);
	  my $scale=$mixed >> 28;
	  my $key=sprintf "%07X", $mixed & 0xFFFFFFF;
	  my $check=spqrcrypt::checkwords($key,$signature);
#	  print "checkwords=$check->[0] $check->[1] $probesignature $key\n";
	  if ( $probesignature ne $check->[0] ) {
	      warn "Authentication failed\n";
	      next;
	  }
          my ($hisport, $hisiaddr) = sockaddr_in($probe);
          my $probeIP = gethostbyaddr($hisiaddr, AF_INET);
	  my $remote = IO::Socket::INET->new(
				    Proto    => "UDP",
				    PeerAddr => $probeIP,
				    PeerPort => $port
				    )
	      || die "socket: $!";
	  my $hispaddr = sockaddr_in($port, $hisiaddr);
	  my $VT = ( $VERSION << 4) | 
	      $packet::messageType{backward};
	  my $backward = pack ("CCnA4NNC*",
			       $VT, 
			       0x0, 
			       $length,
			       $check->[1],
			       tvaltots($scale,$t1tval),  
			       nowts($scale),
			       @data
			       );
	  send ($remote, $backward, 0, $hispaddr);
	  warn "sent backward to $probeIP:$port";
	  last SWITCH; 
      }
      die "Undefined message type\n";
  }
    
}





